package com.nootropic.ioio.rgbmatrix;

import java.io.IOException;
import java.io.OutputStream;

import org.opencv.core.Mat;

import ioio.lib.api.DigitalOutput;
import ioio.lib.api.IOIO;
import ioio.lib.api.IOIOFactory;
import ioio.lib.api.Uart;
import ioio.lib.api.exception.ConnectionLostException;
import android.app.Activity;
import android.os.Bundle;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.Window;

public class RGBMatrixActivity extends Activity {
    private static final String TAG = "RGBDriver::RGBMatrixActivity";
    /** The thread that interacts with the IOIO. */
    private IOIOThread ioioThread;
    private Uart uart = null;
    private OutputStream out = null;
    private RGBMatrixView view;
    public static String debugString = "init";
    
    public static final int VIEW_MODE_9BIT = 0;
    public static final int VIEW_MODE_12BIT = 1;

    private MenuItem mItem9Bit;
    private MenuItem mItem12Bit;

    public static int viewMode = VIEW_MODE_12BIT;

    public RGBMatrixActivity() {
        Log.i(TAG, "Instantiated new " + this.getClass());
    }

    /** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        Log.i(TAG, "onCreate");
        requestWindowFeature(Window.FEATURE_NO_TITLE);
        view = new RGBMatrixView(this);
        setContentView(view);
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        Log.i(TAG, "onCreateOptionsMenu");
        mItem9Bit = menu.add("9-bit color");
        mItem12Bit = menu.add("12-bit color");
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        Log.i(TAG, "Menu Item selected " + item);
        if (item == mItem9Bit) {
            viewMode = VIEW_MODE_9BIT;
        }
        if (item == mItem12Bit) {
            viewMode = VIEW_MODE_12BIT;
        }
        return true;
    }
    
    /**
     * Called when the application is resumed (also when first started). Here is
     * where we'll create our IOIO thread.
     */
    @Override
    protected void onResume() {
        super.onResume();
        ioioThread = new IOIOThread();
        ioioThread.start();
    }

    /**
     * Called when the application is paused. We want to disconnect with the
     * IOIO at this point, as the user is no longer interacting with our
     * application.
     */
    @Override
    protected void onPause() {
        super.onPause();

        ioioThread.abort();
        try {
            ioioThread.join();
        } catch (InterruptedException e) {
        }

        try {
            if (out != null) out.close();
        } catch (IOException e1) {
        }
        if (uart != null) uart.close();

    }

    /**
     * This is the thread that does the IOIO interaction.
     * 
     * It first creates a IOIO instance and wait for a connection to be
     * established. Then it starts doing the main work of opening the LED pin
     * and constantly updating it to match the toggle button's state.
     * 
     * Whenever a connection drops, it tries to reconnect, unless this is a
     * result of abort().
     */
    class IOIOThread extends Thread {
        private IOIO ioio_;
        private DigitalOutput led;
        private boolean abort_ = false;
        Mat outputMat = null;
        int shift = 0;
        int count = 0;

        /** Thread body. */
        @Override
        public void run() {
            super.run();
            while (true) {
                synchronized (this) {
                    if (abort_) {
                        break;
                    }
                    ioio_ = IOIOFactory.create();
                }
                try {
                    debugString = "waiting for connect";
                    ioio_.waitForConnect();
                    debugString = "connected";
                    led = ioio_.openDigitalOutput(0, true);
                    uart = ioio_.openUart(6, 7, 115200, Uart.Parity.NONE, Uart.StopBits.ONE);
                    debugString = "created UART";
                    out = uart.getOutputStream();
                    debugString = "got outputstream";
                    while (outputMat == null) {
                        debugString = "outputMat = null";
                        sleep(100);
                        outputMat = view.getOutputMat();
                    }
                    led.write(true);
                    while (true) {
                        if (viewMode == VIEW_MODE_9BIT) {
                            shift = 5;
                        }
                        if (viewMode == VIEW_MODE_12BIT) {
                            shift = 4;
                        }

                        if ((outputMat.rows() > 0) && (outputMat.cols() > 0)) {
                            debugString = "";
                            led.write(false);
                            // Write the two byte frame marker.
                            out.write(0xF0);
                            out.write(0x00);
                            for (int y = 0; y < outputMat.rows(); y++) {
                                for (int x = 0; x < outputMat.cols(); x++) {
                                    double[] d = outputMat.get(y, x);
                                    // shift each color value to the right to map to a 4-bit color space.
                                    int colorR = ((int) d[0]) >> shift;
                                    int colorG = ((int) d[1]) >> shift;
                                    int colorB = ((int) d[2]) >> shift;
                                    int c1 = colorR & 0xF;  // first byte has red component in lower 4 bits.
                                    int c2 = ((colorG & 0xF) << 4) | (colorB & 0xF);  // second byte has green in upper 4 bits, blue in lower 4 bits.

                                    // Write both bytes for the pixel to the serial output.
                                    out.write(c1);
                                    out.write(c2);
                                }
                            }
                            out.flush();
                        } else {
                            debugString = "outputMat = empty";
                            sleep(100);
                        }
                    }
                } catch (ConnectionLostException e) {
                } catch (Exception e) {
                    debugString = e.toString();
                    try {
                        sleep(5000);
                    } catch (InterruptedException e1) {
                        e1.printStackTrace();
                    }
                    Log.e("RGBMatrixActivity", "Unexpected exception caught", e);
                    ioio_.disconnect();
                    break;
                } finally {
                    try {
                        ioio_.waitForDisconnect();
                    } catch (InterruptedException e) {
                    }
                }
            }
        }

        synchronized public void abort() {
            abort_ = true;
            if (ioio_ != null) {
                ioio_.disconnect();
            }
        }
    }
}
